@@ -3,6 +3,7 @@
  * Copyright (C) 2008 Holger Hans Peter Freyther
  * Copyright (C) 2009 Dirk Schulze <krit@webkit.org>
  * Copyright (C) 2010 Torch Mobile (Beijing) Co. Ltd. All rights reserved.
+ * Copyright (C) 2014 Digia Plc. and/or its subsidiary(-ies)
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,7 +39,6 @@
 #include <wtf/text/WTFString.h>
 
 #include <QBuffer>
-#include <QColor>
 #include <QImage>
 #include <QImageWriter>
 #include <QPainter>
@@ -47,54 +47,21 @@
 
 namespace WebCore {
 
-ImageBufferData::ImageBufferData(const IntSize& size)
-    : m_pixmap(size)
+#if ENABLE(ACCELERATED_2D_CANVAS)
+ImageBuffer::ImageBuffer(const IntSize& size, float /* resolutionScale */, ColorSpace, QOpenGLContext* compatibleContext, bool& success)
+    : m_data(size, compatibleContext)
+    , m_size(size)
+    , m_logicalSize(size)
 {
-    if (m_pixmap.isNull())
+    success = m_data.m_painter && m_data.m_painter->isActive();
+    if (!success)
         return;
 
-    m_pixmap.fill(QColor(Qt::transparent));
-
-    QPainter* painter = new QPainter;
-    m_painter = adoptPtr(painter);
-
-    if (!painter->begin(&m_pixmap))
-        return;
-
-    // Since ImageBuffer is used mainly for Canvas, explicitly initialize
-    // its painter's pen and brush with the corresponding canvas defaults
-    // NOTE: keep in sync with CanvasRenderingContext2D::State
-    QPen pen = painter->pen();
-    pen.setColor(Qt::black);
-    pen.setWidth(1);
-    pen.setCapStyle(Qt::FlatCap);
-    pen.setJoinStyle(Qt::SvgMiterJoin);
-    pen.setMiterLimit(10);
-    painter->setPen(pen);
-    QBrush brush = painter->brush();
-    brush.setColor(Qt::black);
-    painter->setBrush(brush);
-    painter->setCompositionMode(QPainter::CompositionMode_SourceOver);
-    
-    m_image = StillImage::createForRendering(&m_pixmap);
+    m_context = adoptPtr(new GraphicsContext(m_data.m_painter));
 }
+#endif
 
-QImage ImageBufferData::toQImage() const
-{
-    QPaintEngine* paintEngine = m_pixmap.paintEngine();
-    if (!paintEngine || paintEngine->type() != QPaintEngine::Raster)
-        return m_pixmap.toImage();
-
-    // QRasterPixmapData::toImage() will deep-copy the backing QImage if there's an active QPainter on it.
-    // For performance reasons, we don't want that here, so we temporarily redirect the paint engine.
-    QPaintDevice* currentPaintDevice = paintEngine->paintDevice();
-    paintEngine->setPaintDevice(0);
-    QImage image = m_pixmap.toImage();
-    paintEngine->setPaintDevice(currentPaintDevice);
-    return image;
-}
-
-ImageBuffer::ImageBuffer(const IntSize& size, float /* resolutionScale */, ColorSpace, RenderingMode, bool& success)
+ImageBuffer::ImageBuffer(const IntSize& size, float /* resolutionScale */, ColorSpace, RenderingMode /*renderingMode*/, bool& success)
     : m_data(size)
     , m_size(size)
     , m_logicalSize(size)
@@ -103,13 +70,24 @@ ImageBuffer::ImageBuffer(const IntSize& size, float /* resolutionScale */, Color
     if (!success)
         return;
 
-    m_context = adoptPtr(new GraphicsContext(m_data.m_painter.get()));
+    m_context = adoptPtr(new GraphicsContext(m_data.m_painter));
 }
 
 ImageBuffer::~ImageBuffer()
 {
 }
 
+#if ENABLE(ACCELERATED_2D_CANVAS)
+PassOwnPtr<ImageBuffer> ImageBuffer::createCompatibleBuffer(const IntSize& size, float resolutionScale, ColorSpace colorSpace, QOpenGLContext* context)
+{
+    bool success = false;
+    OwnPtr<ImageBuffer> buf = adoptPtr(new ImageBuffer(size, resolutionScale, colorSpace, context, success));
+    if (!success)
+        return nullptr;
+    return buf.release();
+}
+#endif
+
 GraphicsContext* ImageBuffer::context() const
 {
     ASSERT(m_data.m_painter->isActive());
@@ -120,9 +98,9 @@ GraphicsContext* ImageBuffer::context() const
 PassRefPtr<Image> ImageBuffer::copyImage(BackingStoreCopy copyBehavior, ScaleBehavior) const
 {
     if (copyBehavior == CopyBackingStore)
-        return StillImage::create(m_data.m_pixmap);
+        return m_data.m_impl->copyImage();
 
-    return StillImage::createForRendering(&m_data.m_pixmap);
+    return m_data.m_impl->image();
 }
 
 BackingStoreCopy ImageBuffer::fastCopyImageMode()
@@ -131,66 +109,25 @@ BackingStoreCopy ImageBuffer::fastCopyImageMode()
 }
 
 void ImageBuffer::draw(GraphicsContext* destContext, ColorSpace styleColorSpace, const FloatRect& destRect, const FloatRect& srcRect,
-    CompositeOperator op, BlendMode blendMode, bool useLowQualityScale)
+                       CompositeOperator op, BlendMode blendMode, bool useLowQualityScale)
 {
-    if (destContext == context()) {
-        // We're drawing into our own buffer.  In order for this to work, we need to copy the source buffer first.
-        RefPtr<Image> copy = copyImage(CopyBackingStore);
-        destContext->drawImage(copy.get(), ColorSpaceDeviceRGB, destRect, srcRect, op, blendMode, DoNotRespectImageOrientation, useLowQualityScale);
-    } else
-        destContext->drawImage(m_data.m_image.get(), styleColorSpace, destRect, srcRect, op, blendMode, DoNotRespectImageOrientation, useLowQualityScale);
+    m_data.m_impl->draw(destContext, styleColorSpace, destRect, srcRect, op, blendMode, useLowQualityScale, destContext == context());
 }
 
 void ImageBuffer::drawPattern(GraphicsContext* destContext, const FloatRect& srcRect, const AffineTransform& patternTransform,
                               const FloatPoint& phase, ColorSpace styleColorSpace, CompositeOperator op, const FloatRect& destRect)
 {
-    if (destContext == context()) {
-        // We're drawing into our own buffer.  In order for this to work, we need to copy the source buffer first.
-        RefPtr<Image> copy = copyImage(CopyBackingStore);
-        copy->drawPattern(destContext, srcRect, patternTransform, phase, styleColorSpace, op, destRect);
-    } else
-        m_data.m_image->drawPattern(destContext, srcRect, patternTransform, phase, styleColorSpace, op, destRect);
+    m_data.m_impl->drawPattern(destContext, srcRect, patternTransform, phase, styleColorSpace, op, destRect, destContext == context());
 }
 
 void ImageBuffer::clip(GraphicsContext* context, const FloatRect& floatRect) const
 {
-    QPixmap* nativeImage = m_data.m_image->nativeImageForCurrentFrame();
-    if (!nativeImage)
-        return;
-
-    IntRect rect = enclosingIntRect(floatRect);
-    QPixmap alphaMask = *nativeImage;
-
-    context->pushTransparencyLayerInternal(rect, 1.0, alphaMask);
+    m_data.m_impl->clip(context, floatRect);
 }
 
 void ImageBuffer::platformTransformColorSpace(const Vector<int>& lookUpTable)
 {
-    bool isPainting = m_data.m_painter->isActive();
-    if (isPainting)
-        m_data.m_painter->end();
-
-    QImage image = m_data.toQImage().convertToFormat(QImage::Format_ARGB32);
-    ASSERT(!image.isNull());
-
-    uchar* bits = image.bits();
-    const int bytesPerLine = image.bytesPerLine();
-
-    for (int y = 0; y < m_size.height(); ++y) {
-        quint32* scanLine = reinterpret_cast_ptr<quint32*>(bits + y * bytesPerLine);
-        for (int x = 0; x < m_size.width(); ++x) {
-            QRgb& pixel = scanLine[x];
-            pixel = qRgba(lookUpTable[qRed(pixel)],
-                          lookUpTable[qGreen(pixel)],
-                          lookUpTable[qBlue(pixel)],
-                          qAlpha(pixel));
-        }
-    }
-
-    m_data.m_pixmap = QPixmap::fromImage(image);
-
-    if (isPainting)
-        m_data.m_painter->begin(&m_data.m_pixmap);
+    m_data.m_impl->platformTransformColorSpace(lookUpTable);
 }
 
 template <Multiply multiplied>
@@ -207,10 +144,11 @@ PassRefPtr<Uint8ClampedArray> getImageData(const IntRect& rect, const ImageBuffe
     if (rect.x() < 0 || rect.y() < 0 || rect.maxX() > size.width() || rect.maxY() > size.height())
         image.fill(0);
 
-    // Let drawPixmap deal with the conversion.
+    // Let drawImage deal with the conversion.
+    // FIXME: This is inefficient for accelerated ImageBuffers when only part of the imageData is read.
     QPainter painter(&image);
     painter.setCompositionMode(QPainter::CompositionMode_Source);
-    painter.drawPixmap(QPoint(0,0), imageData.m_pixmap, rect);
+    painter.drawImage(QPoint(0,0), imageData.m_impl->toQImage(), rect);
     painter.end();
 
     return result.release();
@@ -233,7 +171,7 @@ void ImageBuffer::putByteArray(Multiply multiplied, Uint8ClampedArray* source, c
 
     bool isPainting = m_data.m_painter->isActive();
     if (!isPainting)
-        m_data.m_painter->begin(&m_data.m_pixmap);
+        m_data.m_painter->begin(m_data.m_impl->paintDevice());
     else {
         m_data.m_painter->save();
 
@@ -278,11 +216,17 @@ String ImageBuffer::toDataURL(const String& mimeType, const double* quality, Coo
     // gif, jpeg..., xpm) so skip the image/ to get the Qt image format used to encode
     // the m_pixmap image.
 
+    RefPtr<Image> image = copyImage(DontCopyBackingStore);
     QByteArray data;
-    if (!encodeImage(m_data.m_pixmap, mimeType.substring(sizeof "image"), quality, data))
+    if (!encodeImage(*image->nativeImageForCurrentFrame(), mimeType.substring(sizeof "image"), quality, data))
         return "data:,";
 
     return "data:" + mimeType + ";base64," + data.toBase64().data();
 }
 
+PlatformLayer* ImageBuffer::platformLayer() const
+{
+    return m_data.m_impl->platformLayer();
+}
+
 }
