@@ -24,16 +24,17 @@
 #include "GraphicsLayerAnimation.h"
 #include "GraphicsLayerFactory.h"
 #include "ImageBuffer.h"
-#include "NotImplemented.h"
 #include <wtf/CurrentTime.h>
 
-#if USE(CAIRO)
-#include "CairoUtilities.h"
-#include <wtf/text/CString.h>
-#endif
+#if USE(TEXTURE_MAPPER)
 
 namespace WebCore {
 
+TextureMapperLayer* toTextureMapperLayer(GraphicsLayer* layer)
+{
+    return layer ? toGraphicsLayerTextureMapper(layer)->layer() : 0;
+}
+
 PassOwnPtr<GraphicsLayer> GraphicsLayer::create(GraphicsLayerFactory* factory, GraphicsLayerClient* client)
 {
     if (!factory)
@@ -44,8 +45,6 @@ PassOwnPtr<GraphicsLayer> GraphicsLayer::create(GraphicsLayerFactory* factory, G
 
 PassOwnPtr<GraphicsLayer> GraphicsLayer::create(GraphicsLayerClient* client)
 {
-    if (s_graphicsLayerFactory)
-        return (*s_graphicsLayerFactory)(client);
     return adoptPtr(new GraphicsLayerTextureMapper(client));
 }
 
@@ -53,17 +52,17 @@ GraphicsLayerTextureMapper::GraphicsLayerTextureMapper(GraphicsLayerClient* clie
     : GraphicsLayer(client)
     , m_layer(adoptPtr(new TextureMapperLayer()))
     , m_compositedNativeImagePtr(0)
-    , m_changeMask(0)
+    , m_changeMask(NoChanges)
     , m_needsDisplay(false)
-    , m_hasOwnBackingStore(true)
     , m_fixedToViewport(false)
     , m_debugBorderWidth(0)
     , m_contentsLayer(0)
-    , m_animationStartedTimer(this, &GraphicsLayerTextureMapper::animationStartedTimerFired)
+    , m_animationStartTime(0)
+    , m_isScrollable(false)
 {
 }
 
-void GraphicsLayerTextureMapper::notifyChange(TextureMapperLayer::ChangeMask changeMask)
+void GraphicsLayerTextureMapper::notifyChange(ChangeMask changeMask)
 {
     m_changeMask |= changeMask;
     if (!client())
@@ -78,6 +77,9 @@ void GraphicsLayerTextureMapper::setName(const String& name)
 
 GraphicsLayerTextureMapper::~GraphicsLayerTextureMapper()
 {
+    if (m_contentsLayer)
+        m_contentsLayer->setClient(0);
+
     willBeDestroyed();
 }
 
@@ -90,11 +92,11 @@ void GraphicsLayerTextureMapper::willBeDestroyed()
 */
 void GraphicsLayerTextureMapper::setNeedsDisplay()
 {
-    if (!m_hasOwnBackingStore)
+    if (!drawsContent())
         return;
 
     m_needsDisplay = true;
-    notifyChange(TextureMapperLayer::DisplayChange);
+    notifyChange(DisplayChange);
     addRepaintRect(FloatRect(FloatPoint(), m_size));
 }
 
@@ -102,7 +104,7 @@ void GraphicsLayerTextureMapper::setNeedsDisplay()
 */
 void GraphicsLayerTextureMapper::setContentsNeedsDisplay()
 {
-    notifyChange(TextureMapperLayer::DisplayChange);
+    notifyChange(DisplayChange);
     addRepaintRect(contentsRect());
 }
 
@@ -110,37 +112,32 @@ void GraphicsLayerTextureMapper::setContentsNeedsDisplay()
 */
 void GraphicsLayerTextureMapper::setNeedsDisplayInRect(const FloatRect& rect)
 {
-    if (!m_hasOwnBackingStore)
+    if (!drawsContent())
         return;
 
     if (m_needsDisplay)
         return;
     m_needsDisplayRect.unite(rect);
-    notifyChange(TextureMapperLayer::DisplayChange);
+    notifyChange(DisplayChange);
     addRepaintRect(rect);
 }
 
-/* \reimp (GraphicsLayer.h)
-*/
-void GraphicsLayerTextureMapper::setParent(GraphicsLayer* layer)
-{
-    notifyChange(TextureMapperLayer::ParentChange);
-    GraphicsLayer::setParent(layer);
-}
-
 /* \reimp (GraphicsLayer.h)
 */
 bool GraphicsLayerTextureMapper::setChildren(const Vector<GraphicsLayer*>& children)
 {
-    notifyChange(TextureMapperLayer::ChildrenChange);
-    return GraphicsLayer::setChildren(children);
+    if (GraphicsLayer::setChildren(children)) {
+        notifyChange(ChildrenChange);
+        return true;
+    }
+    return false;
 }
 
 /* \reimp (GraphicsLayer.h)
 */
 void GraphicsLayerTextureMapper::addChild(GraphicsLayer* layer)
 {
-    notifyChange(TextureMapperLayer::ChildrenChange);
+    notifyChange(ChildrenChange);
     GraphicsLayer::addChild(layer);
 }
 
@@ -149,15 +146,15 @@ void GraphicsLayerTextureMapper::addChild(GraphicsLayer* layer)
 void GraphicsLayerTextureMapper::addChildAtIndex(GraphicsLayer* layer, int index)
 {
     GraphicsLayer::addChildAtIndex(layer, index);
-    notifyChange(TextureMapperLayer::ChildrenChange);
+    notifyChange(ChildrenChange);
 }
 
 /* \reimp (GraphicsLayer.h)
 */
 void GraphicsLayerTextureMapper::addChildAbove(GraphicsLayer* layer, GraphicsLayer* sibling)
 {
-     GraphicsLayer::addChildAbove(layer, sibling);
-     notifyChange(TextureMapperLayer::ChildrenChange);
+    GraphicsLayer::addChildAbove(layer, sibling);
+    notifyChange(ChildrenChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -165,7 +162,7 @@ void GraphicsLayerTextureMapper::addChildAbove(GraphicsLayer* layer, GraphicsLay
 void GraphicsLayerTextureMapper::addChildBelow(GraphicsLayer* layer, GraphicsLayer* sibling)
 {
     GraphicsLayer::addChildBelow(layer, sibling);
-    notifyChange(TextureMapperLayer::ChildrenChange);
+    notifyChange(ChildrenChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -173,22 +170,12 @@ void GraphicsLayerTextureMapper::addChildBelow(GraphicsLayer* layer, GraphicsLay
 bool GraphicsLayerTextureMapper::replaceChild(GraphicsLayer* oldChild, GraphicsLayer* newChild)
 {
     if (GraphicsLayer::replaceChild(oldChild, newChild)) {
-        notifyChange(TextureMapperLayer::ChildrenChange);
+        notifyChange(ChildrenChange);
         return true;
     }
     return false;
 }
 
-/* \reimp (GraphicsLayer.h)
-*/
-void GraphicsLayerTextureMapper::removeFromParent()
-{
-    if (!parent())
-        return;
-    notifyChange(TextureMapperLayer::ParentChange);
-    GraphicsLayer::removeFromParent();
-}
-
 /* \reimp (GraphicsLayer.h)
 */
 void GraphicsLayerTextureMapper::setMaskLayer(GraphicsLayer* value)
@@ -196,7 +183,7 @@ void GraphicsLayerTextureMapper::setMaskLayer(GraphicsLayer* value)
     if (value == maskLayer())
         return;
     GraphicsLayer::setMaskLayer(value);
-    notifyChange(TextureMapperLayer::MaskLayerChange);
+    notifyChange(MaskLayerChange);
 
     if (!value)
         return;
@@ -212,7 +199,7 @@ void GraphicsLayerTextureMapper::setReplicatedByLayer(GraphicsLayer* value)
     if (value == replicaLayer())
         return;
     GraphicsLayer::setReplicatedByLayer(value);
-    notifyChange(TextureMapperLayer::ReplicaLayerChange);
+    notifyChange(ReplicaLayerChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -222,7 +209,7 @@ void GraphicsLayerTextureMapper::setPosition(const FloatPoint& value)
     if (value == position())
         return;
     GraphicsLayer::setPosition(value);
-    notifyChange(TextureMapperLayer::PositionChange);
+    notifyChange(PositionChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -232,7 +219,7 @@ void GraphicsLayerTextureMapper::setAnchorPoint(const FloatPoint3D& value)
     if (value == anchorPoint())
         return;
     GraphicsLayer::setAnchorPoint(value);
-    notifyChange(TextureMapperLayer::AnchorPointChange);
+    notifyChange(AnchorPointChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -245,7 +232,7 @@ void GraphicsLayerTextureMapper::setSize(const FloatSize& value)
     GraphicsLayer::setSize(value);
     if (maskLayer())
         maskLayer()->setSize(value);
-    notifyChange(TextureMapperLayer::SizeChange);
+    notifyChange(SizeChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -256,7 +243,7 @@ void GraphicsLayerTextureMapper::setTransform(const TransformationMatrix& value)
         return;
 
     GraphicsLayer::setTransform(value);
-    notifyChange(TextureMapperLayer::TransformChange);
+    notifyChange(TransformChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -266,7 +253,7 @@ void GraphicsLayerTextureMapper::setChildrenTransform(const TransformationMatrix
     if (value == childrenTransform())
         return;
     GraphicsLayer::setChildrenTransform(value);
-    notifyChange(TextureMapperLayer::ChildrenTransformChange);
+    notifyChange(ChildrenTransformChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -276,7 +263,7 @@ void GraphicsLayerTextureMapper::setPreserves3D(bool value)
     if (value == preserves3D())
         return;
     GraphicsLayer::setPreserves3D(value);
-    notifyChange(TextureMapperLayer::Preserves3DChange);
+    notifyChange(Preserves3DChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -286,7 +273,7 @@ void GraphicsLayerTextureMapper::setMasksToBounds(bool value)
     if (value == masksToBounds())
         return;
     GraphicsLayer::setMasksToBounds(value);
-    notifyChange(TextureMapperLayer::MasksToBoundsChange);
+    notifyChange(MasksToBoundsChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -295,8 +282,11 @@ void GraphicsLayerTextureMapper::setDrawsContent(bool value)
 {
     if (value == drawsContent())
         return;
-    notifyChange(TextureMapperLayer::DrawsContentChange);
     GraphicsLayer::setDrawsContent(value);
+    notifyChange(DrawsContentChange);
+
+    if (value)
+        setNeedsDisplay();
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -305,7 +295,7 @@ void GraphicsLayerTextureMapper::setContentsVisible(bool value)
 {
     if (value == contentsAreVisible())
         return;
-    notifyChange(TextureMapperLayer::ContentsVisibleChange);
+    notifyChange(ContentsVisibleChange);
     GraphicsLayer::setContentsVisible(value);
     if (maskLayer())
         maskLayer()->setContentsVisible(value);
@@ -317,7 +307,7 @@ void GraphicsLayerTextureMapper::setContentsOpaque(bool value)
 {
     if (value == contentsOpaque())
         return;
-    notifyChange(TextureMapperLayer::ContentsOpaqueChange);
+    notifyChange(ContentsOpaqueChange);
     GraphicsLayer::setContentsOpaque(value);
 }
 
@@ -328,7 +318,7 @@ void GraphicsLayerTextureMapper::setBackfaceVisibility(bool value)
     if (value == backfaceVisibility())
         return;
     GraphicsLayer::setBackfaceVisibility(value);
-    notifyChange(TextureMapperLayer::BackfaceVisibilityChange);
+    notifyChange(BackfaceVisibilityChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -338,7 +328,7 @@ void GraphicsLayerTextureMapper::setOpacity(float value)
     if (value == opacity())
         return;
     GraphicsLayer::setOpacity(value);
-    notifyChange(TextureMapperLayer::OpacityChange);
+    notifyChange(OpacityChange);
 }
 
 /* \reimp (GraphicsLayer.h)
@@ -348,9 +338,19 @@ void GraphicsLayerTextureMapper::setContentsRect(const IntRect& value)
     if (value == contentsRect())
         return;
     GraphicsLayer::setContentsRect(value);
-    notifyChange(TextureMapperLayer::ContentsRectChange);
+    notifyChange(ContentsRectChange);
 }
 
+void GraphicsLayerTextureMapper::setContentsToSolidColor(const Color& color)
+{
+    if (color == m_solidColor)
+        return;
+
+    m_solidColor = color;
+    notifyChange(BackgroundColorChange);
+}
+
+
 /* \reimp (GraphicsLayer.h)
 */
 void GraphicsLayerTextureMapper::setContentsToImage(Image* image)
@@ -376,7 +376,7 @@ void GraphicsLayerTextureMapper::setContentsToImage(Image* image)
     }
 
     setContentsToMedia(m_compositedImage.get());
-    notifyChange(TextureMapperLayer::ContentChange);
+    notifyChange(ContentChange);
     GraphicsLayer::setContentsToImage(image);
 }
 
@@ -386,62 +386,227 @@ void GraphicsLayerTextureMapper::setContentsToMedia(TextureMapperPlatformLayer*
         return;
 
     GraphicsLayer::setContentsToMedia(media);
-    notifyChange(TextureMapperLayer::ContentChange);
+    notifyChange(ContentChange);
+
+    if (m_contentsLayer)
+        m_contentsLayer->setClient(0);
+
     m_contentsLayer = media;
+
+    if (m_contentsLayer)
+        m_contentsLayer->setClient(this);
+}
+
+void GraphicsLayerTextureMapper::setShowDebugBorder(bool show)
+{
+    if (isShowingDebugBorder() == show)
+        return;
+
+    GraphicsLayer::setShowDebugBorder(show);
+    notifyChange(DebugVisualsChange);
+}
+
+void GraphicsLayerTextureMapper::setShowRepaintCounter(bool show)
+{
+    if (isShowingRepaintCounter() == show)
+        return;
+
+    GraphicsLayer::setShowRepaintCounter(show);
+    notifyChange(DebugVisualsChange);
+}
+
+void GraphicsLayerTextureMapper::didCommitScrollOffset(const IntSize& offset)
+{
+    if (offset.isZero())
+        return;
+
+    m_committedScrollOffset = offset;
+    notifyChange(CommittedScrollOffsetChange);
+}
+
+void GraphicsLayerTextureMapper::setIsScrollable(bool isScrollable)
+{
+    if (m_isScrollable == isScrollable)
+        return;
+
+    m_isScrollable = isScrollable;
+    notifyChange(IsScrollableChange);
 }
 
 /* \reimp (GraphicsLayer.h)
 */
 void GraphicsLayerTextureMapper::flushCompositingStateForThisLayerOnly()
 {
-    m_layer->flushCompositingState(this);
-    didFlushCompositingState();
+    prepareBackingStoreIfNeeded();
+    commitLayerChanges();
+    m_layer->syncAnimations();
+    updateBackingStoreIfNeeded();
+}
+
+void GraphicsLayerTextureMapper::prepareBackingStoreIfNeeded()
+{
+    if (!shouldHaveBackingStore()) {
+        m_backingStore.clear();
+        m_changeMask |= BackingStoreChange;
+    } else {
+        if (!m_backingStore) {
+            m_backingStore = TextureMapperTiledBackingStore::create();
+            m_changeMask |= BackingStoreChange;
+        }
+    }
+
+    updateDebugBorderAndRepaintCount();
+}
+
+void GraphicsLayerTextureMapper::updateDebugBorderAndRepaintCount()
+{
+    if (isShowingDebugBorder())
+        updateDebugIndicators();
+
+    // When this has its own backing store (e.g. Qt WK1), update the repaint count before calling TextureMapperLayer::flushCompositingStateForThisLayerOnly().
+    bool needsToRepaint = shouldHaveBackingStore() && (m_needsDisplay || !m_needsDisplayRect.isEmpty());
+    if (isShowingRepaintCounter() && needsToRepaint) {
+        incrementRepaintCount();
+        m_changeMask |= RepaintCountChange;
+    }
+}
+
+void GraphicsLayerTextureMapper::setDebugBorder(const Color& color, float width)
+{
+    m_debugBorderColor = color;
+    m_debugBorderWidth = width;
+    m_changeMask |= DebugVisualsChange;
+}
+
+static void toTextureMapperLayerVector(const Vector<GraphicsLayer*>& layers, Vector<TextureMapperLayer*>& texmapLayers)
+{
+    texmapLayers.reserveCapacity(layers.size());
+    for (size_t i = 0; i < layers.size(); ++i)
+        texmapLayers.append(toTextureMapperLayer(layers[i]));
+}
+
+void GraphicsLayerTextureMapper::commitLayerChanges()
+{
+    if (m_changeMask == NoChanges)
+        return;
+
+    if (m_changeMask & ChildrenChange) {
+        Vector<TextureMapperLayer*> textureMapperLayerChildren;
+        toTextureMapperLayerVector(children(), textureMapperLayerChildren);
+        m_layer->setChildren(textureMapperLayerChildren);
+    }
+
+    if (m_changeMask & MaskLayerChange)
+        m_layer->setMaskLayer(toTextureMapperLayer(maskLayer()));
+
+    if (m_changeMask & ReplicaLayerChange)
+        m_layer->setReplicaLayer(toTextureMapperLayer(replicaLayer()));
+
+    if (m_changeMask & PositionChange)
+        m_layer->setPosition(position());
+
+    if (m_changeMask & AnchorPointChange)
+        m_layer->setAnchorPoint(anchorPoint());
+
+    if (m_changeMask & SizeChange)
+        m_layer->setSize(size());
+
+    if (m_changeMask & TransformChange)
+        m_layer->setTransform(transform());
+
+    if (m_changeMask & ChildrenTransformChange)
+        m_layer->setChildrenTransform(childrenTransform());
+
+    if (m_changeMask & Preserves3DChange)
+        m_layer->setPreserves3D(preserves3D());
+
+    if (m_changeMask & ContentsRectChange)
+        m_layer->setContentsRect(contentsRect());
+
+    if (m_changeMask & MasksToBoundsChange)
+        m_layer->setMasksToBounds(masksToBounds());
+
+    if (m_changeMask & DrawsContentChange)
+        m_layer->setDrawsContent(drawsContent());
+
+    if (m_changeMask & ContentsVisibleChange)
+        m_layer->setContentsVisible(contentsAreVisible());
+
+    if (m_changeMask & ContentsOpaqueChange)
+        m_layer->setContentsOpaque(contentsOpaque());
+
+    if (m_changeMask & BackfaceVisibilityChange)
+        m_layer->setBackfaceVisibility(backfaceVisibility());
+
+    if (m_changeMask & OpacityChange)
+        m_layer->setOpacity(opacity());
+
+    if (m_changeMask & BackgroundColorChange)
+        m_layer->setSolidColor(solidColor());
+
+#if ENABLE(CSS_FILTERS)
+    if (m_changeMask & FilterChange)
+        m_layer->setFilters(filters());
+#endif
+
+    if (m_changeMask & BackingStoreChange)
+        m_layer->setBackingStore(m_backingStore);
+
+    if (m_changeMask & DebugVisualsChange)
+        m_layer->setDebugVisuals(isShowingDebugBorder(), debugBorderColor(), debugBorderWidth(), isShowingRepaintCounter());
+
+    if (m_changeMask & RepaintCountChange)
+        m_layer->setRepaintCount(repaintCount());
+
+    if (m_changeMask & ContentChange)
+        m_layer->setContentsLayer(platformLayer());
+
+    if (m_changeMask & AnimationChange)
+        m_layer->setAnimations(m_animations);
+
+    if (m_changeMask & AnimationStarted)
+        client()->notifyAnimationStarted(this, m_animationStartTime);
+
+    if (m_changeMask & FixedToViewporChange)
+        m_layer->setFixedToViewport(fixedToViewport());
+
+    if (m_changeMask & IsScrollableChange)
+        m_layer->setIsScrollable(isScrollable());
+
+    if (m_changeMask & CommittedScrollOffsetChange)
+        m_layer->didCommitScrollOffset(m_committedScrollOffset);
+
+    m_changeMask = NoChanges;
 }
 
 /* \reimp (GraphicsLayer.h)
 */
-void GraphicsLayerTextureMapper::flushCompositingState(const FloatRect&)
+void GraphicsLayerTextureMapper::flushCompositingState(const FloatRect& rect)
 {
-    m_layer->flushCompositingState(this, TextureMapperLayer::TraverseDescendants);
-    didFlushCompositingStateRecursive();
-}
-
-void GraphicsLayerTextureMapper::didFlushCompositingState()
-{
-    updateBackingStore();
-    m_changeMask = 0;
-}
-
-void GraphicsLayerTextureMapper::didFlushCompositingStateRecursive()
-{
-    didFlushCompositingState();
-    for (size_t i = 0; i < children().size(); ++i)
-        toGraphicsLayerTextureMapper(children()[i])->didFlushCompositingStateRecursive();
-    if (maskLayer())
-        toGraphicsLayerTextureMapper(maskLayer())->didFlushCompositingStateRecursive();
-    if (replicaLayer())
-        toGraphicsLayerTextureMapper(replicaLayer())->didFlushCompositingStateRecursive();
-}
-
-void GraphicsLayerTextureMapper::updateBackingStore()
-{
-    if (!m_hasOwnBackingStore)
+    if (!m_layer->textureMapper())
         return;
 
-    prepareBackingStore();
-    m_layer->setBackingStore(m_backingStore);
+    flushCompositingStateForThisLayerOnly();
+
+    if (maskLayer())
+        maskLayer()->flushCompositingState(rect);
+    if (replicaLayer())
+        replicaLayer()->flushCompositingState(rect);
+    for (size_t i = 0; i < children().size(); ++i)
+        children()[i]->flushCompositingState(rect);
 }
 
-void GraphicsLayerTextureMapper::prepareBackingStore()
+void GraphicsLayerTextureMapper::updateBackingStoreIfNeeded()
 {
     TextureMapper* textureMapper = m_layer->textureMapper();
     if (!textureMapper)
         return;
 
     if (!shouldHaveBackingStore()) {
-        m_backingStore.clear();
+        ASSERT(!m_backingStore);
         return;
     }
+    ASSERT(m_backingStore);
 
     IntRect dirtyRect = enclosingIntRect(FloatRect(FloatPoint::zero(), m_size));
     if (!m_needsDisplay)
@@ -449,31 +614,12 @@ void GraphicsLayerTextureMapper::prepareBackingStore()
     if (dirtyRect.isEmpty())
         return;
 
-    if (!m_backingStore)
-        m_backingStore = TextureMapperTiledBackingStore::create();
-
-    // Paint the entire dirty rect into an image buffer. This ensures we only paint once.
-    OwnPtr<ImageBuffer> imageBuffer = ImageBuffer::create(dirtyRect.size());
-    GraphicsContext* context = imageBuffer->context();
-    context->setImageInterpolationQuality(textureMapper->imageInterpolationQuality());
-    context->setTextDrawingMode(textureMapper->textDrawingMode());
-    context->translate(-dirtyRect.x(), -dirtyRect.y());
-    paintGraphicsLayerContents(*context, dirtyRect);
-
-    if (isShowingRepaintCounter()) {
-        incrementRepaintCount();
-        drawRepaintCounter(context);
-    }
-
-    RefPtr<Image> image = imageBuffer->copyImage(DontCopyBackingStore);
-#if PLATFORM(QT)
+#if PLATFORM(QT) && !defined(QT_NO_DYNAMIC_CAST)
     ASSERT(dynamic_cast<TextureMapperTiledBackingStore*>(m_backingStore.get()));
 #endif
     TextureMapperTiledBackingStore* backingStore = static_cast<TextureMapperTiledBackingStore*>(m_backingStore.get());
-    backingStore->updateContents(textureMapper, image.get(), m_size, dirtyRect, BitmapTexture::UpdateCanModifyOriginalImageData);
 
-    backingStore->setShowDebugBorders(isShowingDebugBorder());
-    backingStore->setDebugBorder(m_debugBorderColor, m_debugBorderWidth);
+    backingStore->updateContents(textureMapper, this, m_size, dirtyRect, BitmapTexture::UpdateCanModifyOriginalImageData);
 
     m_needsDisplay = false;
     m_needsDisplayRect = IntRect();
@@ -484,40 +630,6 @@ bool GraphicsLayerTextureMapper::shouldHaveBackingStore() const
     return drawsContent() && contentsAreVisible() && !m_size.isEmpty();
 }
 
-#if USE(CAIRO)
-void GraphicsLayerTextureMapper::drawRepaintCounter(GraphicsContext* context)
-{
-    cairo_t* cr = context->platformContext()->cr();
-    cairo_save(cr);
-
-    CString repaintCount = String::format("%i", this->repaintCount()).utf8();
-    cairo_select_font_face(cr, "sans-serif", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
-    cairo_set_font_size(cr, 18);
-
-    cairo_text_extents_t repaintTextExtents;
-    cairo_text_extents(cr, repaintCount.data(), &repaintTextExtents);
-
-    static const int repaintCountBorderWidth = 10;
-    setSourceRGBAFromColor(cr, isShowingDebugBorder() ? m_debugBorderColor : Color(0, 255, 0, 127));
-    cairo_rectangle(cr, 0, 0,
-        repaintTextExtents.width + (repaintCountBorderWidth * 2),
-        repaintTextExtents.height + (repaintCountBorderWidth * 2));
-    cairo_fill(cr);
-
-    cairo_set_source_rgb(cr, 1, 1, 1);
-    cairo_move_to(cr, repaintCountBorderWidth, repaintTextExtents.height + repaintCountBorderWidth);
-    cairo_show_text(cr, repaintCount.data());
-
-    cairo_restore(cr);
-}
-#else
-void GraphicsLayerTextureMapper::drawRepaintCounter(GraphicsContext* context)
-{
-    notImplemented();
-}
-
-#endif
-
 bool GraphicsLayerTextureMapper::addAnimation(const KeyframeValueList& valueList, const IntSize& boxSize, const Animation* anim, const String& keyframesName, double timeOffset)
 {
     ASSERT(!keyframesName.isEmpty());
@@ -531,16 +643,22 @@ bool GraphicsLayerTextureMapper::addAnimation(const KeyframeValueList& valueList
     if (valueList.property() == AnimatedPropertyWebkitTransform)
         listsMatch = validateTransformOperations(valueList, hasBigRotation) >= 0;
 
-    m_animations.add(GraphicsLayerAnimation(keyframesName, valueList, boxSize, anim, WTF::currentTime() - timeOffset, listsMatch));
-    notifyChange(TextureMapperLayer::AnimationChange);
-    m_animationStartedTimer.startOneShot(0);
+    const double currentTime = WTF::currentTime();
+    m_animations.add(GraphicsLayerAnimation(keyframesName, valueList, boxSize, anim, currentTime - timeOffset, listsMatch));
+    // m_animationStartTime is the time of the first real frame of animation, now or delayed by a negative offset.
+    if (timeOffset > 0)
+        m_animationStartTime = currentTime;
+    else
+        m_animationStartTime = currentTime - timeOffset;
+    notifyChange(AnimationChange);
+    notifyChange(AnimationStarted);
     return true;
 }
 
 void GraphicsLayerTextureMapper::setAnimations(const GraphicsLayerAnimations& animations)
 {
     m_animations = animations;
-    notifyChange(TextureMapperLayer::AnimationChange);
+    notifyChange(AnimationChange);
 }
 
 
@@ -554,25 +672,32 @@ void GraphicsLayerTextureMapper::removeAnimation(const String& animationName)
     m_animations.remove(animationName);
 }
 
-void GraphicsLayerTextureMapper::animationStartedTimerFired(Timer<GraphicsLayerTextureMapper>*)
-{
-    client()->notifyAnimationStarted(this, /* DOM time */ WTF::currentTime());
-}
-
-void GraphicsLayerTextureMapper::setDebugBorder(const Color& color, float width)
-{
-    // The default values for GraphicsLayer debug borders are a little
-    // hard to see (some less than one pixel wide), so we double their size here.
-    m_debugBorderColor = color;
-    m_debugBorderWidth = width * 2;
-}
-
 #if ENABLE(CSS_FILTERS)
 bool GraphicsLayerTextureMapper::setFilters(const FilterOperations& filters)
 {
-    notifyChange(TextureMapperLayer::FilterChange);
+    TextureMapper* textureMapper = m_layer->textureMapper();
+    // TextureMapperImageBuffer does not support CSS filters.
+    if (!textureMapper || textureMapper->accelerationMode() == TextureMapper::SoftwareMode)
+        return false;
+    notifyChange(FilterChange);
     return GraphicsLayer::setFilters(filters);
 }
 #endif
 
+void GraphicsLayerTextureMapper::setFixedToViewport(bool fixed)
+{
+    if (m_fixedToViewport == fixed)
+        return;
+
+    m_fixedToViewport = fixed;
+    notifyChange(FixedToViewporChange);
 }
+
+void GraphicsLayerTextureMapper::setRepaintCount(int repaintCount)
+{
+    m_repaintCount = repaintCount;
+    notifyChange(RepaintCountChange);
+}
+
+}
+#endif
